from uuid import UUID

from db.repositories.order import OrderRepository
from db.repositories.product import ProductRepository
from .schemas import OrderGetSchema, OrderListSchema, OrderStatusSchema
from exceptions import common as common_exc, http as http_exc
from db.models import ProductOrder, Order


class OrderService:
    order_repository = OrderRepository()
    product_repository = ProductRepository()

    async def create_order(self, **kwargs):
        total = 0
        order = await self.order_repository.create(**kwargs)

        order_products = []

        for item in kwargs.get('items'):
            try:
                product = await self.product_repository.get(id=item['id'])
            except common_exc.NotFoundException:
                raise http_exc.HTTPBadRequestException(detail='Object product does not exist')

            quantity = item['quantity']

            await ProductOrder.create(order=order, product=product, quantity=quantity)

            total += product.price * quantity

            order_products.append({"id": product.id, "price": product.price, "quantity": quantity})

        order.total = total
        await order.save()

        order_data = {
            "id": order.id,
            "status": order.status,
            "total": order.total,
            "products": order_products
        }

        return OrderGetSchema.model_validate(order_data)

    async def accepted_order(self, id: UUID):
        try:
            order = await self.order_repository.get(id=id)
        except common_exc.NotFoundException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

        order.status = Order.StatusEnum.CONFIRMED
        await order.save()

        return {'id': order.id, 'status': order.status}

    async def get_order_status(self, id: UUID):
        order = await self.order_repository.get(id)
        return {'id': order.id, 'status': order.status}

    async def get_order_list(self, **kwargs):
        orders = await self.order_repository.get_list(**kwargs)

        return {
            "page": orders['page'],
            "size": orders['size'],
            "total": orders['total'],
            "result": [
                OrderListSchema.model_validate(
                    {
                        'id': order.id, 'status': order.status, 'total': order.total,
                        'created_at': order.created_at, 'updated_at': order.updated_at
                     }
                ) for order in orders['result']
            ]
        }
